<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Post_Overview extends WordPress_Multilingual_Multisite {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The ID of this plugin.
     */
    protected $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of this plugin.
     */
    protected $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version, $network_query, $translation_tables ) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->network_query = $network_query;

        $this->translation_tables = $translation_tables;
        $this->translation_tables->set_table('posts_translations');

        $this->prefix = 'wordpress_multilingual_multisite_';

    }

    public function init() {
        global $wordpress_multilingual_multisite;

        $this->options = $wordpress_multilingual_multisite;

        if (!$this->get_option('enable'))
        {
            return FALSE;
        }
    }

    /**
     * Add custom ticket metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @param   [type]                       $post_type [description]
     * @param   [type]                       $post      [description]
     */
    public function add_network_pages()
    {

        add_menu_page( 'Post Translations', 'Post Translations', 'manage_network_options', 'post-translations', array($this, 'add_menu'), 'dashicons-admin-post');

        $post_types = $this->translation_tables->get_post_types();
        if(empty($post_types)) {
            return;
        }

        foreach ($post_types as $post_type) {

            if(!isset($post_type['type']) || empty($post_type['type'])) {
                continue;
            }

            $post_type = $post_type['type'];
            $post_type_pbject = get_post_type_object($post_type);
            if(!$post_type_pbject) {
                continue;
            }

            $post_type_name = $post_type_pbject->labels->name;
            if(!$post_type_pbject) {
                $post_type_name = ucfirst( $post_type );
            } else {
                $post_type_name = $post_type_pbject->labels->name;
            }

            add_submenu_page('post-translations', $post_type_name, $post_type_name, 'manage_network_options', $post_type, array($this, 'add_menu'));
        }      
    }

    public function add_menu()
    {
        global $blogs_data;

        $post_type = $_GET['page'];
        $query_blog_id = isset($_GET['blog_id']) && !empty($_GET['blog_id']) ? $_GET['blog_id'] : 1;

        if(!$post_type) {
            return;
        }

        $items = $this->translation_tables->get_translated_items_by_type($post_type, array_keys($blogs_data));

        if(empty($items)) {
            echo 'No items found';
            return false;
        }

        echo '<h1>' . ucfirst($post_type) . ' Connected Translations</h1>';

        echo '<div class="wrap">';

            echo '<table class="wp-list-table widefat fixed striped">';
                

                $first = true;

                foreach ($items as $blog_id => $items) {


                    if($first) {

                        echo '<tr>';
                        foreach ($items as $blog_id => $post_id) { 
                            
                                $country = strtolower($blogs_data[$blog_id]['country']);
                                $language = $blogs_data[$blog_id]['language'];

                                echo '<td><span class="flag-icon flag-icon-' . $country . '"></span> (' . $language . '-' . $country . ')</td>';
                            
                            $first = false;
                        }
                        echo '</tr>';
                    }

                    echo '<tr>';

                        foreach ($items as $blog_id => $post_id) {

                            switch_to_blog($blog_id);


                            $post = get_post($post_id);
                            if(!$post) {
                                echo '<td>N/A</td>';
                                continue;
                            }

                            $editLink = ' <a href="' . $blogs_data[$blog_id]['admin_url'] . '/post.php?post=' . $post_id . '&action=edit" target="_blank"><i class="fa fa-edit"></i></a>';
                            $viewLink = ' <a href="' . $post->guid . '" target="_blank"><i class="fa fa-eye"></i></a>';

                            echo '<td>' . $post->post_title . $viewLink . $editLink .' (ID: ' . $post->ID . ')' . '</td>';

                            restore_current_blog();        
                         }

                    echo '</tr>';



                }

            echo '</table>';
        echo '</div>';

        restore_current_blog();
    }

    public function _old_add_menu()
    {
        global $blogs_data;

        $post_type = get_post_type_object($_GET['page']);
        $query_blog_id = isset($_GET['blog_id']) && !empty($_GET['blog_id']) ? $_GET['blog_id'] : 1;
        $current_url = network_admin_url('admin.php?page=' . $post_type->name);

        echo '<h1>' . $post_type->labels->name . '</h1>';

        echo '<ul class="subsubsub">';
        foreach ($blogs_data as $blog_id => $blog_data) {
            $current = "";
            if($query_blog_id == $blog_id) {
                $current = 'class="current"';
            }
            echo '<li><a href="' . $current_url . '&blog_id=' .  $blog_id . '" ' . $current . '>' . $blog_data['name'] . ' (' . $blog_data['locale'] . ')</a> | </li>';
        }
        echo '</ul>';
        echo '<div class="clear"></div>';

        switch_to_blog($query_blog_id);

        $args = array(
            'post_type' => $post_type->name,
        );

        $posts = get_posts($args);

        if(empty($posts)) {
            echo sprintf( esc_html__('No %s found ...', 'wordpress-multilingual-multisite'), $post_type->labels->name);
            return false;
        }

        $data_to_show = array(
            'post_title' => esc_html__('Title', 'wordpress-multilingual-multisite'), 
            'post_status' => esc_html__('Status', 'wordpress-multilingual-multisite'), 
        );

        $data_to_show = apply_filters('wordpress_multilingual_multisite_post_overview_keys_to_show', $data_to_show);

        unset($blogs_data[$query_blog_id]);

        ?>

        <div class="wrap">

            <table class="wp-list-table widefat fixed striped">
                <thead>
                    <tr>
                        <?php 
                        foreach ($data_to_show as $thead) {
                            echo '<th class="col">' . $thead . '</th>';
                        }
                        
                        foreach ($blogs_data as $blog_id => $blog_data) {
                            $country = $blogs_data[$blog_id]['country'];
                             echo '<th class="col"><span class="flag-icon flag-icon-' . $country . '"></span> ' . $blog_data['name'] . ' (' . $blog_data['locale'] . ')</th>';
                        }
                        ?>
                    </tr>
                </thead>
                <tbody>
                    <?php 
                    foreach ($posts as $post) {

                        $translated = get_post_meta($post->ID, 'translated_posts', true);

                        echo '<tr>';

                            echo '<td>' . $post->post_title . '</td>';
                            echo '<td>' . $post->post_status . '</td>';

                            foreach ($blogs_data as $blog_id => $blog_data) {
                                

                                if(isset($translated[$blog_id])) {
                                    echo 
                                    '<td>
                                        <a href="' . get_blog_permalink($blog_id, $translated[$blog_id]) . '">
                                            View
                                        </a>
                                    </td>';
                                } else {
                                     echo 
                                     '<td>
                                        <a href="' . get_blog_permalink($blog_id, $translated[$blog_id]) . '">
                                            Create
                                        </a>
                                    </td>';
                                }
                            }

                        echo '</tr>';
                    }
                    ?>
                </tbody>
            </table>

        </div>
        <?php

        restore_current_blog();
    }
}