<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Posts_Translations extends WordPress_Multilingual_Multisite {

	protected $plugin_name;
	protected $version;
    protected $current_blog_id;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $network_query, $translation_tables, $current_blog_id ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
        $this->network_query = $network_query;
        
        $this->translation_tables = $translation_tables;

        $this->prefix = 'wordpress_multilingual_multisite_';
        $this->current_blog_id = $current_blog_id;

	}

    public function init() 
    {
        global $wordpress_multilingual_multisite, $blogs_data;
        $this->options = $wordpress_multilingual_multisite;

        if (!$this->get_option('enable')) {
            return FALSE;
        }

        if(!$this->get_option('showConnectedItemsInOverview')) {
            return FALSE;
        }
        
        if(count($blogs_data) > 14) {
            return FALSE;
        }

        add_filter('manage_posts_columns', array($this, 'modify_columns_head') );
        add_action('manage_posts_custom_column', array($this, 'modify_columns_content') , 10, 2);

        $postTypes = get_post_types();
        unset($postTypes['post']);
        foreach($postTypes as $postType) {

            add_filter('manage_' . $postType . '_posts_columns', array($this, 'modify_columns_head') , 10);
            add_action('manage_' . $postType . '_posts_custom_column', array($this, 'modify_columns_content') , 10, 2);
        }
    }

    /**
     * Add custom ticket metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @param   [type]                       $post_type [description]
     * @param   [type]                       $post      [description]
     */
    public function add_custom_metaboxes($post_type, $post)
    {
        add_meta_box('wordpress-multilingual-multisite-connected', esc_html__('Connected Posts', 'wordpress-multilingual-multisite'), array($this, 'connected'), '', 'side', 'high');
    }

    /**
     * Display Metabox metabox
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @return  [type]                       [description]
     */
    public function connected()
    {
        global $post, $blogs_data;

        // $args = array(
        //     'post_type' => $post->post_type,
        //     'post_status' => 'any',
        //     'sites_not' => $this->current_blog_id,
        //     'posts_per_page'   => -1,
        //     'orderby'=> 'title', 
        //     'order' => 'ASC'
        // );

        // $posts_by_site =  $this->network_query->get_posts( $args );

        wp_nonce_field(basename(__FILE__), 'wordpress_multilingual_multisite_meta_nonce');

        $this->translation_tables->set_table('posts_translations');
        $translated = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $post->ID);
        $excludedSites = $this->get_option('excludedSites');

        echo '<div class="wordpress-multilingual-multisite-container">';

            if($this->get_option('syncOnlySpecificPosts')) {

                $checked = "";
                $syncThisPost = get_post_meta($post->ID, 'wordpress_multilingual_multisite_sync_post', true);
                if($syncThisPost == "1") {
                    $checked = 'checked="checked"';
                }

                echo '<label for="wordpress-multilingual-multisite-sync-post" style="margin-bottom: 10px; display:block; margin-top: 10px; border-bottom: 1px solid #eaeaea; padding-bottom: 5px;">
                <input id="wordpress-multilingual-multisite-sync-post" type="checkbox" name="wordpress_multilingual_multisite_sync_post" value="1" ' . $checked . '>
                ' . esc_html__('Sync this post', 'wordpress-multilingual-multisite') . '
                </label>';

            }

            echo '<input type="hidden" name="translated_posts[' . $this->current_blog_id . ']" value="' . $post->ID . '">';
            echo '<input type="hidden" name="translated_posts[type]" value="' . $post->post_type . '">';

            // if(!empty($posts_by_site)) {
                foreach ($blogs_data as $blog_id => $blog_data) {

                    // if($excludedSites && in_array($blog_id, $excludedSites)) {
                    //     continue;
                    // }
                    if($this->current_blog_id == $blog_id) {
                        continue;
                    }

                    $country = strtolower($blog_data['country']);
                    $options = '';
                    $link = $blog_data['admin_url'] . '?wp-trans-copy=post&source_blog=' . $this->current_blog_id . '&source_id=' . $post->ID;
                    
                    if(isset($translated[$blog_id]) && !empty($translated[$blog_id]) ) {

                        $translatedPostId = $translated[$blog_id];
                        $translatedPost = get_blog_post($blog_id, $translatedPostId);
                        $link = $blog_data['admin_url'] . 'post.php?action=edit&post=' . $translatedPostId;

                        $options .= '<option value="' . $translatedPostId . '" selected="selected">' . $translatedPost->post_title . ' (ID: ' . $translatedPost->ID . ' / Status '  . $translatedPost->post_status . ')' . '</option>';    
                    }
                    

                    ?>

                    <div class="wordpress-multilingual-multisite-row">
                        <div class="wordpress-multilingual-multisite-col-sm-2">
                            <a href="<?php echo $link ?>"><span class="flag-icon flag-icon-<?php echo $country ?>"></span></a>
                        </div>

                        <div class="wordpress-multilingual-multisite-col-sm-10">

                            <select name="translated_posts[<?php echo $blog_id ?>]" data-blog-id="<?php echo $blog_id ?>" data-post-type="<?php echo $post->post_type ?>" class="wordpress-multilingual-multisite-input-field wordpress-multilingual-multisite-translated-posts-select">
                                <option value="0"></option>
                                <?php echo $options ?>
                            </select>
                        </div>
                    </div>
                    <?php
                }
            // }


        echo '</div>';
    }

    /**
     * Save Custom Metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @param   [type]                       $post_id [description]
     * @param   [type]                       $post    [description]
     * @return  [type]                                [description]
     */
    public function save_custom_metaboxes($post_id, $post, $update)
    {
        global $post, $sync_running;

        if(!is_a($post, 'WP_Post')) {
            return false;
        }

        if (!current_user_can('edit_post', $post->ID)) {
            return $post->ID;
        }

        if ($post->post_type == 'revision') {
            return false;
        }

        if (!isset($_POST['wordpress_multilingual_multisite_meta_nonce']) || !wp_verify_nonce($_POST['wordpress_multilingual_multisite_meta_nonce'], basename(__FILE__))) {
            return false;
        }

        if($sync_running) {
            return;
        }

        if($this->get_option('syncOnlySpecificPosts')) {
            if(isset($_POST['wordpress_multilingual_multisite_sync_post']) && !empty($_POST['wordpress_multilingual_multisite_sync_post'])) {
                update_post_meta($post->ID, 'wordpress_multilingual_multisite_sync_post', esc_html($_POST['wordpress_multilingual_multisite_sync_post']) );
            } else {
                delete_post_meta($post->ID, 'wordpress_multilingual_multisite_sync_post' );
            }
        }

        if(!isset($_POST['translated_posts']) || empty($_POST['translated_posts'])) {
            return flase;
        }



        $this->translation_tables->set_table('posts_translations');
        $translated_posts = $_POST['translated_posts'];

        $exists = $this->translation_tables->get_translated_items_by_translated_items($translated_posts);
        if($exists) {
            $this->translation_tables->update_translated_items($exists['id'], $this->current_blog_id, $post->ID, $translated_posts);
        } else {
            $this->translation_tables->insert_translated_items($this->current_blog_id, $post->ID, $translated_posts);
        }

        delete_site_transient('wordpress_multilingual_multisite_connected_posts');

    }

    public function modify_columns_head($defaults) 
    {
        global $blogs_data;

        // $excludedSites = $this->get_option('excludedSites');
        // if($excludedSites && in_array($this->current_blog_id, $excludedSites)) {
        //     return $defaults;
        // }

        foreach ($blogs_data as $blog_id => $blog_data) {
            if($blog_id == $this->current_blog_id) {
                continue;
            }

            $defaults['blog_' . $blog_id] = '<span class="flag-icon flag-icon-' . strtolower($blog_data['country']) . '"></span><br><span style="display: none;">' . $blog_data['country'] . '-' . $blog_data['language'] . '</span>';
        }

        return $defaults;
    }

    public function modify_columns_content($column_name, $post_ID) 
    {
        global $blogs_data;

        // $excludedSites = $this->get_option('excludedSites');
        // if($excludedSites && in_array($this->current_blog_id, $excludedSites)) {
        //     return $column_name;
        // }

        $this->translation_tables->set_table('posts_translations');
        $translated = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $post_ID);        

        if(substr($column_name, 0, 5) == "blog_") {
            $blog_id = substr($column_name, 5);

            if(isset($translated[$blog_id])) {

                $link = $blogs_data[$blog_id]['admin_url'] . 'post.php?action=edit&post=' . $translated[$blog_id];
                $html = '<a href="' . $link . '"><i class="fa fa-pen-square fa-2x"></i></a>';
            } else {
                $post_type = get_post_type($post_ID);
                if ($this->get_option('postCopierEnable')) {
                    $link = $blogs_data[$blog_id]['admin_url'] . '?wp-trans-copy=post&source_blog=' . $this->current_blog_id . '&source_id=' . $post_ID;
                } else {
                    $link = $blogs_data[$blog_id]['admin_url'] . 'post-new.php?post_type=' . $post_type;
                }
                
                $html = '<a href="' . $link . '"><i class="fa fa-plus-square fa-2x"></i></a>';
            }
            echo wp_kses_post($html);
        }        
    }

    public function init_translation_tables()
    {
        if(!is_admin()) {
            return false;
        }

        global $blogs_data;
        $first_blog_id = array_key_first($blogs_data);
        if(empty($first_blog_id)) {
            return false;
        }
        $excludedSites = $this->get_option('excludedSites');

        switch_to_blog($first_blog_id);

        global $wpdb;

        // Posts
        $results = $wpdb->get_results("SELECT ID, post_type FROM {$wpdb->prefix}posts WHERE `post_type` NOT IN ('revision', 'custom_css', 'customize_changeset', 'oembed_cache', 'vc4_templates', 'vc_grid_item', 'product_variation', 'shop_order_refund' );", ARRAY_A);

        if(empty($results)) {
            return false;
        }

        $this->translation_tables->set_table('posts_translations');

        foreach ($results as $result) {

            $post_id = $result['ID'];

            $result[$first_blog_id] = $post_id;
            $result['type'] = $result['post_type'];

            unset($result['ID']);
            unset($result['post_type']);
            
            foreach($blogs_data as $blog_id => $blog_data) {

                if( $first_blog_id == $blog_id) {
                    continue;
                }

                if($excludedSites && in_array($blog_id, $excludedSites)) {
                    continue;
                }
            }

            $exists = $this->translation_tables->get_translated_items_by_translated_items($result);
            if($exists) {
                $this->translation_tables->update_translated_items($exists['id'], $first_blog_id, $post_id, $result);
            } else {
                $this->translation_tables->insert_translated_items($first_blog_id, $post_id, $result);
            }
        }

        echo count($results) . ' Posts from Blog ID: ' . $first_blog_id . ' have been inserted into posts_translations table.';

        // Terms
        $results = $wpdb->get_results("SELECT term_id FROM {$wpdb->prefix}terms;", ARRAY_A);
        if(empty($results)) {
            return false;
        }

        $this->translation_tables->set_table('terms_translations');
        foreach ($results as $result) {

            $term_id = $result['term_id'];
            $result[$first_blog_id] = $term_id;

            foreach($blogs_data as $blog_id => $blog_data) {

                if( $first_blog_id == $blog_id) {
                    continue;
                }

                if($excludedSites && in_array($blog_id, $excludedSites)) {
                    continue;
                }

                $result[$blog_id] = $term_id;
            }

            unset($result['term_id']);
        
            $exists = $this->translation_tables->get_translated_items_by_translated_items($result);
            if($exists) {
                $this->translation_tables->update_translated_items($exists['id'], $first_blog_id, $term_id, $result);
            } else {
                $this->translation_tables->insert_translated_items($first_blog_id, $term_id, $result);
            }
        }

        echo '<br>' . count($results) . ' Terms from Blog ID: ' . $first_blog_id . ' have been inserted into terms_translations table.';
        die();
    }

    public function ajax_get_network_posts()
    {
        $response = array();

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            die(json_encode($response));
        }

        $blog_id = intval($_POST['blog_id']);
        $post_type = esc_attr($_POST['post_type']);
        $search = esc_attr($_POST['search']);
        if(empty($blog_id) || empty($post_type) || empty($search)) {
            die(json_encode($response));
        }

        switch_to_blog($blog_id);

        $queryArgs = array(
            'post_type' => $post_type,
            'numberposts' => -1,
            's' => $search,
            'post_status' => 'any',
        );

        $posts = get_posts($queryArgs);

        restore_current_blog();

        if(!empty($posts)) {

            $response['items'] = array();
            $response['items'][] = array(
                'id' => 0,
                'text' => 'None',
            );

            foreach($posts as $post) {
                $response['items'][] = array(
                    'id' => $post->ID,
                    'text' => $post->post_title . ' (ID: ' . $post->ID . ')',
                );
            }
        }

        die(json_encode($response));

    }

}