<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Public extends WordPress_Multilingual_Multisite {

	protected $plugin_name;
	protected $version;
	protected $current_blog_id;
	protected $connected_links = array();

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $translation_tables, $current_blog_id) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;

		$this->translation_tables = $translation_tables;

		$this->current_blog_id = $current_blog_id;
	}

    /**
     * Enqueue Admin Styles
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://welaunch.io/plugins/wordpress-multilingual-multisite/
     * @return  boolean
     */
    public function enqueue_styles()
    {
    	global $blogs_data;
    	if(!isset($blogs_data[$this->current_blog_id])) {
    		return;
    	}

        wp_enqueue_style($this->plugin_name . '-public', plugin_dir_url( dirname( __FILE__ ) ).'assets/css/wordpress-multilingual-multisite-public.css', array(), $this->version, 'all');
        wp_enqueue_style($this->plugin_name . '-flag-icon', plugin_dir_url( dirname( __FILE__ ) ).'assets/css/flag-icon-css/css/flag-icon.min.css', array(), '3.1.0', 'all');
    }

    /**
     * Enqueue Admin Scripts
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://welaunch.io/plugins/wordpress-multilingual-multisite/
     * @return  boolean
     */
    public function enqueue_scripts()
    {
        
    }


	public function init() 
    {
		global $wordpress_multilingual_multisite, $blogs_data;

		$this->options = $wordpress_multilingual_multisite;

		if (!$this->get_option('enable')) {
			return FALSE;
		}

		if(is_admin()) {
			return;
		}

		$this->maybe_redirect();
		$this->get_connected();

		add_shortcode('wordpress_multilingual_multisite_country_selector', array($this, 'get_country_selector'));
		add_shortcode('wordpress_multilingual_multisite_country_selector_page', array($this, 'get_country_selector_page'));

	}

	public function maybe_redirect()
	{
		if(is_admin() || defined( 'DOING_AJAX' ) && DOING_AJAX ) {
			return false;
		}

		if(isset($_POST) && !empty($_POST)) {
			return false;
		}
		
		if( current_user_can( 'administrator' ) ){
			return false;
		}

		if(!$this->get_option('masterAutomaticRedirect')) {
			return false;
		}

		$masterSite = $this->get_option('masterSite');
		if(empty($masterSite)) {
			return false;
		}

		if($this->current_blog_id != $masterSite) {
			return false;
		}

		global $blogs_data, $post;
		if(count($blogs_data) == 1 && array_key_first($blogs_data) == $this->current_blog_id) {
			return false;
		}

	    $is_bot = $this->detect_bots();
	    if($is_bot) {
	        return false;
	    }
		
		$countryCode = "";
		$excludedSites = $this->get_option('excludedSites');
		$redirectURL = "";

		$userLangCountry = $this->get_users_lang_and_country();

		$redirectBlog = false;
		$geolocate = false;

		$userIP = $_SERVER["REMOTE_ADDR"];

		// IP based redirect
		if($this->get_option('masterAutomaticRedirectIP') && !empty($userIP)) {

			// $apiKey = $this->get_option('masterAutomaticRedirectIPAPIKey');
            $url = "https://get.geojs.io/v1/ip/geo/" . $userIP . '.json';

            $geolocate = json_decode(file_get_contents($url));

            if(isset($geolocate->country_code)) {

                $countryCode = $geolocate->country_code;
                if(strlen($userLangCountry) == 5) {
                	$userLangCountry = substr($userLangCountry, 0, 2) . '-' . $countryCode;
            	} else {
            		$userLangCountry .= '-' . $countryCode;
            	}

            	// Redirect just on Country (not recommended cause in many country there are multiple languages spoken)
        		if($this->get_option('masterAutomaticRedirectIPCountryFirst')) {

					// Try Country
					foreach ($blogs_data as $blog_id => $blog_data) {

				        if($excludedSites && in_array($blog_id, $excludedSites)) {
				            continue;
				        }

						if( $blog_data['country'] == $countryCode) {
							$redirectBlog = $blog_data;
							break;
						}
					}
        		}
            }
		}

		if(empty($redirectBlog)) {
			if(strlen($userLangCountry) == 5) {

				// Try Lang and Country
				foreach ($blogs_data as $blog_id => $blog_data) {

			        if($excludedSites && in_array($blog_id, $excludedSites)) {
			            continue;
			        }

					if( ($blog_data['language'] . '-' . $blog_data['country']) == $userLangCountry) {
						$redirectBlog = $blog_data;
						break;
					}
				}

				$userLangCountry = substr($userLangCountry, 0, 2);
				if(empty($redirectBlog)) {
					foreach ($blogs_data as $blog_id => $blog_data) {

				        if($excludedSites && in_array($blog_id, $excludedSites)) {
				            continue;
				        }

						if($blog_data['language'] == $userLangCountry) {
							$redirectBlog = $blog_data;
							break;
						}
					}
				}

			} else {
				// Try only Lang
				foreach ($blogs_data as $blog_id => $blog_data) {

			        if($excludedSites && in_array($blog_id, $excludedSites)) {
			            continue;
			        }

					if($blog_data['language'] == $userLangCountry) {
						$redirectBlog = $blog_data;
						break;
					}
				}
			}
		}

		$defaultRedirect = $this->get_option('masterDefaultRedirect');
		if(!empty($defaultRedirect) && empty($redirectBlog)) {
			$redirectBlog = $blogs_data[$defaultRedirect];
		}

		if(empty($redirectBlog)) {
			return false;
		}

		if($redirectBlog['blog_id'] == $this->current_blog_id) {
			return false;
		}

		if($this->get_option('masterAutomaticRedirectShortLinks')) {

			if(is_category() || is_tag() || is_tax()) {
				$this->get_connected_term_links();
			} elseif(is_single() || is_page() || is_front_page()) {
				$this->get_connected_post_links();
			}

			if(!empty($this->connected_links) && isset($this->connected_links[$redirectBlog['blog_id']]) && !empty($this->connected_links[$redirectBlog['blog_id']])) {
				$redirectURL = $this->connected_links[$redirectBlog['blog_id']];
			}
		} 
		
		if(empty($redirectURL)) {
			$redirectURL = $redirectBlog['site_url'];
		}

		if(empty($redirectURL)) {
			return false;
		}

		if(isset($_GET['debug'])) {
			var_dump($geolocate);
			var_dump($userLangCountry);
			var_dump($redirectBlog);
			die();
		}

		header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");
		header("Cache-Control: post-check=0, pre-check=0", false);
		header("Pragma: no-cache");
		header("HTTP/1.1 302 Moved Temporary");
	    header("Location: " . $redirectURL);
	}

    /**
     * Detect Bots
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     */
    private function detect_bots()
    {
        if (isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/bot|google|baidu|bing|msn|duckduckbot|teoma|slurp|yandex|crawl|spider/i', $_SERVER['HTTP_USER_AGENT'])) {
            return TRUE;
        } else {
            return FALSE;
        }
    }

	public function get_users_lang_and_country()
	{
		$user_lang = "en";

		if(!isset($_SERVER["HTTP_ACCEPT_LANGUAGE"]) || empty($_SERVER["HTTP_ACCEPT_LANGUAGE"])) {
			return $user_lang;
		}

        $languages = explode(",", $_SERVER["HTTP_ACCEPT_LANGUAGE"]);

        foreach($languages as $language)
        {
            $lang = explode(';', $language);
            $lang = $lang[0];

            $user_lang = $lang;
            break;

        }

        return $user_lang;
	}

	public function add_alternate_tags()
	{
		global $blogs_data;

		if(empty($this->connected_links)) {
			return false;
		}

		foreach ($this->connected_links as $blog_id => $connected_link) {
			if(!isset($blogs_data[$blog_id])) {
				continue;
			}

			echo '<link rel="alternate" hreflang="' . $blogs_data[$blog_id]['alternate'] . '" href="' . $connected_link . '" />';
			echo "\n";
		}
	}

	public function get_country_selector($atts = array())
	{
		global $blogs_data, $wp_trans_languages, $wp_trans_countries;

        $attributes = extract( shortcode_atts( array(
            'text' => $this->get_option('countrySelectorText'),
            'caret' => $this->get_option('countrySelectorShowCaret'),
            'showCurrent' => $this->get_option('countrySelectorShowCurrentFlag'),
        ), $atts ) );

        if(!isset($blogs_data[get_current_blog_id()]) || empty($blogs_data[get_current_blog_id()])) {
        	return "";
        }

        $currentSite = $blogs_data[get_current_blog_id()];
        $currentSiteName = $currentSite['name'];
		if(isset($currentSite['country_selector_name']) && !empty($currentSite['country_selector_name'])) {
			$currentSiteName = $currentSite['country_selector_name'];
		}
		
		if(isset($currentSite['country_selector_flag_url']) && !empty($currentSite['country_selector_flag_url'])) {
			$current_flag = '<img src="' . $currentSite['country_selector_flag_url'] . '" class="wordpress-multilingual-multisite-country-selector-flag" />';
        } else {
	        if($showCurrent) {
	        	$current_flag = '<span class="flag-icon flag-icon-' . strtolower($currentSite['country']) . '"></span>';
	        } else {
	        	$current_flag = '<span class="flag-icon flag-icon-un"></span>';
	        }        	
        }

    	if(!isset($currentSite['language']) || empty($currentSite['language'])) {
    		$language = "en";
    	} else {
    		$language = $currentSite['language'];
    	}

    	if(!isset($currentSite['country']) || empty($currentSite['country'])) {
    		$country = "en";
    	} else {
    		$country = $currentSite['country'];
    	}

        $text = str_replace(
		        	array('{{language}}', '{{country}}', '{{blog_name}}'), 
		        	array($wp_trans_languages[$language], $wp_trans_countries[$country], $currentSiteName), 
		        	$text
		        );

        $html = '
        <a class="country_selector_dropdown">
            <div class="country_selector_dropbtn">' . $current_flag . $text;

	        if($caret) {
	        	$html .= ' <i class="fas fa-caret-down"></i>';
	        }

        	$html .= '</div>
    	</a>';

        $html .= '
            <div class="wordpress-multilingual-multisite-country">
	        ' . $this->get_country_selector_ul() . '
	        </div>';
        

        return $html;
	}

	public function get_country_selector_ul()
	{
		global $blogs_data, $wp_trans_languages, $wp_trans_countries;

		$countrySelectorDropdownText = $this->get_option('countrySelectorDropdownText');

    	if(!isset($blogs_data[$this->current_blog_id]) ) {
    		return;
    	}

    	$excludedSites = $this->get_option('excludedSites');

		$lis = "";
        foreach ($blogs_data as $blog_id => $blog_data) {

	        if($excludedSites && in_array($blog_id, $excludedSites)) {
	            continue;
	        }

        	if(isset($this->connected_links[$blog_id])) {
        		$link =  $this->connected_links[$blog_id];
        	} elseif($this->get_option('countrySelectorShowWhenNotLinked')) {
        		$link = $blog_data['site_url'];
        	} else {
        		continue;
        	}

        	if(!isset($blog_data['language']) || empty($blog_data['language'])) {
        		$language = "en";
        	} else {
        		$language = $blog_data['language'];
        	}

        	if(!isset($blog_data['country']) || empty($blog_data['country'])) {
        		$country = "UN";
        	} else {
        		$country = $blog_data['country'];
        	}

	        $blogName = $blog_data['name'];
			if(isset($blog_data['country_selector_name']) && !empty($blog_data['country_selector_name'])) {
				$blogName = $blog_data['country_selector_name'];
			}

        	$text = str_replace(
		        	array('{{language}}', '{{country}}', '{{blog_name}}'), 
		        	array($wp_trans_languages[$language], 
		        		$wp_trans_countries[$country], 
		        		$blogName), 
		        	$countrySelectorDropdownText
	        );

        	$flag = '<span class="flag-icon flag-icon-' . strtolower($country) . '"></span>';
			if(isset($blog_data['country_selector_flag_url']) && !empty($blog_data['country_selector_flag_url'])) {
				$flag = '<img src="' . $blog_data['country_selector_flag_url'] . '" class="wordpress-multilingual-multisite-country-selector-flag" />';
	        }

        	
        	$lis .= '<li><a href="' . $link . '">' . $flag . $text . '</a></li>';
        }
        return sprintf('<ul class="wordpress-multilingual-multisite-country-selector">%s</ul>', $lis);
	}

	public function get_country_selector_page()
	{
        $ul = $this->get_country_selector_ul();
        return $ul;
	}

    protected function get_connected()
    {
		global $blogs_data;

		if(is_category() || is_tag() || is_tax()) {
			$this->get_connected_term_links();
		} elseif(is_single() || is_page() || is_front_page()) {
			$this->get_connected_post_links();
		} elseif(function_exists('is_shop') && is_shop() && function_exists('is_product_category') && !is_product_category()) {
		    $this->get_woocommerce_specific_links('shop');
		} else {
			$this->get_fallback_replace();
		}
    }

    protected function get_connected_term_links()
    {
    	global $blogs_data;

    	$term_id = get_queried_object_id();

    	$filtered = false;
		if( (function_exists('flrt_is_filter_request') && flrt_is_filter_request()) || function_exists('flrt_is_filter_request') && is_filtered()) {
			$filtered = true;
		}

    	$connected_terms_transient = get_site_transient('wordpress_multilingual_multisite_connected_terms');
    	if(!$filtered && isset($connected_terms_transient[$this->current_blog_id]) && isset($connected_terms_transient[$this->current_blog_id][$term_id])) {
    		$this->connected_links = $connected_terms_transient[$this->current_blog_id][$term_id];
    		return;
    	}

		$this->translation_tables->set_table('terms_translations');
		$translations_exists = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $term_id);

		$term = get_term( $term_id );
		$original_base = $this->get_base_for_taxonomy($term->taxonomy);

		foreach ($blogs_data as $blog_id => $blog_data) {

			if(isset($translations_exists[$blog_id])) {
				switch_to_blog($blog_id);

				$base = $this->get_base_for_taxonomy($term->taxonomy, $blog_id);

				if($base) {
					$link = str_replace($original_base, $base, get_category_link($translations_exists[$blog_id]));
				} else {
					$link = get_category_link($translations_exists[$blog_id]);
				}

				if(isset($translations_exists[$blog_id])) {
					$this->connected_links[$blog_id] = $link;
				}

				restore_current_blog();
			}
		}

		if(!isset($connected_terms_transient[$this->current_blog_id])) {
			$connected_terms_transient[$this->current_blog_id] = array();
		}

		$this->connected_links = apply_filters('wordpress_multilingual_multisite_connected_term_links', $this->connected_links, $term_id, $filtered);

		if(!$filtered) {
			$connected_terms_transient[$this->current_blog_id][$term_id] = $this->connected_links;
			set_site_transient('wordpress_multilingual_multisite_connected_terms', $connected_terms_transient);
		}
    }

    protected function get_fallback_replace()
    {
    	global $blogs_data, $post;

    	if(!isset($blogs_data[$this->current_blog_id])) {
    		return false;
    	}
    	
    	$current_url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://" . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
    	$current_blog_url = $blogs_data[$this->current_blog_id]['site_url'];
    	
		foreach ($blogs_data as $blog_id => $blog_data) {
			$link = str_replace($current_blog_url, $blog_data['site_url'], $current_url);
			$this->connected_links[$blog_id] = $link;
		}
    }

    protected function get_base_for_taxonomy($taxonomy, $blog_id = false)
    {
    	if(!$blog_id) {
    		$blog_id = $this->current_blog_id;
    	}

    	$base = "";
    	$default = "";
    	
    	switch ($taxonomy) {
    		case 'post_tag':
				$base = get_option('tag_base');
    			break;
    		case 'product_cat':
				$base = get_option('woocommerce_permalinks');
				$base = isset($base['category_base']) ? $base['category_base'] : false;
				if(!$base) {
					$default = 'product-category';	
				}
    			break;
    		case 'product_tag':
				$base = get_option('woocommerce_permalinks');
				$base = isset($base['tag_base']) ? $base['tag_base'] : false;
				if(!$base) {
					$default = 'product-tag';	
				}
    			break;
    		case 'product_attribute':
				$base = get_option('woocommerce_permalinks');
				$base = isset($base['attribute_base']) ? $base['attribute_base'] : false;
				if(!$base) {
					$default = '';	
				}
    			break;
			case 'product':
				$base = get_option('woocommerce_permalinks');
				$base = isset($base['product_base']) ? $base['product_base'] : false;
				if(!$base) {
					$default = '/product';	
				}
    			break;
				break;
    		default:
    			global $wp_rewrite;

    			if(isset($wp_rewrite->extra_permastructs[$taxonomy]) && !empty($wp_rewrite->extra_permastructs[$taxonomy])) {
    				$base = preg_replace('/%[\s\S]+?%/', '', $wp_rewrite->extra_permastructs[$taxonomy]['struct'] );
    			} else {
    				$base = get_option($taxonomy . '_base');
    			}

    			$base = apply_filters('wordpress_multilingual_multisite_taxonomy_base', $base, $taxonomy, $blog_id);

    			break;
    	}

    	if(!$base && !$default) {
    		$base = $taxonomy;
    	}

    	if(!$base) {
    		$base = $default;
    	}

    	return $base;
    }

    protected function get_connected_post_links()
    {
    	global $blogs_data, $post;

    	$connected_posts_transient = get_site_transient('wordpress_multilingual_multisite_connected_posts');
    	if(isset($connected_posts_transient[$this->current_blog_id]) && isset($connected_posts_transient[$this->current_blog_id][$post->ID])) {
    		$this->connected_links = $connected_posts_transient[$this->current_blog_id][$post->ID];
    		return;
    	}

		$this->translation_tables->set_table('posts_translations');
		$translations_exists = $this->translation_tables->get_translated_items_by_item_id($this->current_blog_id, $post->ID);
		$excludedSites = $this->get_option('excludedSites');

		if($this->get_option('countrySelectorProductSKUFallback') && in_array($post->post_type, array('product', 'product_variation')) ) {
			$sku = get_post_meta($post->ID, '_sku', true);
		}
			
		if($post->post_type != "page") {
			$original_base = $this->get_base_for_taxonomy($post->post_type);
		}

		foreach ($blogs_data as $blog_id => $blog_data) {
			
	        if($excludedSites && in_array($blog_id, $excludedSites)) {
	            continue;
	        }

	        switch_to_blog($blog_id);

	        if(!isset($translations_exists[$blog_id]) && 
	        	!empty($sku) && 
		        function_exists('wc_get_product_id_by_sku') && 
		        $this->get_option('countrySelectorProductSKUFallback') && 
		        in_array($post->post_type, array('product', 'product_variation')) ) 
	        {
	        	$skuExists = wc_get_product_id_by_sku($sku);
				if(!empty($skuExists)) {
					$translations_exists[$blog_id] = $skuExists;
				}
        	}

			if(isset($translations_exists[$blog_id]) ) {
				if($post->post_type != "page" && !is_front_page()) {

					$base = $this->get_base_for_taxonomy($post->post_type, $blog_id);
					if($base) {
						$this->connected_links[$blog_id] = str_replace($original_base, $base, get_blog_permalink($blog_id, $translations_exists[$blog_id]));
					} else {
						$this->connected_links[$blog_id] = get_blog_permalink($blog_id, $translations_exists[$blog_id]);
					}

					
				} else {
					$this->connected_links[$blog_id] = get_blog_permalink($blog_id, $translations_exists[$blog_id]);
				}
			}

			restore_current_blog();
		}

		if(!isset($connected_posts_transient[$this->current_blog_id])) {
			$connected_posts_transient[$this->current_blog_id] = array();
		}

		$connected_posts_transient[$this->current_blog_id][$post->ID] = $this->connected_links;
		set_site_transient('wordpress_multilingual_multisite_connected_posts', $connected_posts_transient);
    }

    protected function get_woocommerce_specific_links($type = 'shop')
    {
    	global $blogs_data;

    	foreach ($blogs_data as $blog_id => $blog_data) {

    		switch_to_blog($blog_id);

    		$pageId = get_option( 'woocommerce_' . $type . '_page_id' );

    		if(!empty($pageId)) {
    			$link = get_permalink($pageId);
    			$this->connected_links[$blog_id] = $link;
    		}

    		restore_current_blog();
		}
    }

    public function add_country_selector_to_menu($items, $args)
    {
    	global $blogs_data;

    	if(!$this->get_option('countrySelectorMenuEnable')) {
    		return $items;
    	}

    	if(!isset($blogs_data[$this->current_blog_id]) ) {
    		return $items;
    	}

    	$menu_location = $this->get_option('countrySelectorMenuThemeLocation');
    	if(empty($menu_location)) {
    		return $items;
    	}

    	$menu_location = explode(',', $menu_location);

    	if( in_array($args->theme_location, $menu_location) ) {
    		$items .= '<li class="menu-item menu-item-type-post_type menu-item-object-post wordpress-multilingual-multisite-country-selector-in-menu">' . $this->get_country_selector() . '' .
    		'</li>';
    	}

    	return $items;
    }

    public function change_language_attributes($output, $doctype)
   	{

		global $blogs_data;
    	if(!isset($blogs_data[$this->current_blog_id]) ) {
    		return $output;
    	}

		$attributes = array();
	 
	    if ( function_exists( 'is_rtl' ) && is_rtl() ) {
	        $attributes[] = 'dir="rtl"';
	    }
	 
	    $lang = $blogs_data[$this->current_blog_id]['alternate'];
	    if(empty($lang)) {
			return $output;
	    }

	    if ( $lang ) {
	        if ( 'text/html' === get_option( 'html_type' ) || 'html' === $doctype ) {
	            $attributes[] = 'lang="' . esc_attr( $lang ) . '"';
	        }
	 
	        if ( 'text/html' !== get_option( 'html_type' ) || 'xhtml' === $doctype ) {
	            $attributes[] = 'xml:lang="' . esc_attr( $lang ) . '"';
	        }
	    }
	 
	    $output = implode( ' ', $attributes );
	 
	    /**
	     * Filters the language attributes for display in the 'html' tag.
	     *
	     * @since 2.5.0
	     * @since 4.3.0 Added the `$doctype` parameter.
	     *
	     * @param string $output A space-separated list of language attributes.
	     * @param string $doctype The type of HTML document (xhtml|html).
	     */
	    return $output;
    }
}