<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-multilingual-multisite/
 * @since      1.0.0
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WordPress_Multilingual_Multisite
 * @subpackage WordPress_Multilingual_Multisite/admin
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Multilingual_Multisite_Translation_Tables extends WordPress_Multilingual_Multisite {

	protected $plugin_name;
	protected $version;
	protected $current_blog_id;
	protected $table;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $current_blog_id) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->current_blog_id = $current_blog_id;
	}

	public function init() 
	{

		global $wordpress_multilingual_multisite;

		$this->options = $wordpress_multilingual_multisite;

		if (!$this->get_option('enable'))
		{
			return FALSE;
		}
	}

	public function set_table($table = 'items_translations')
	{
		global $wpdb;

		$this->table = $wpdb->base_prefix . $table;
	}

	/**
	 * [add_blog_to_translation_table description]
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $blog_id [description]
	 * @param   [type]                       $user_id [description]
	 * @param   [type]                       $domain  [description]
	 * @param   [type]                       $path    [description]
	 * @param   [type]                       $site_id [description]
	 * @param   [type]                       $meta    [description]
	 */
	public function add_blog_to_translation_table( $site ) 
	{
	   	global $wpdb;

	   	$blog_id = $site->blog_id;

	   	$tables = array('posts_translations', 'terms_translations');
	   	foreach ($tables as $table) {
		   	$table_name = $wpdb->base_prefix . $table;

			$sql = "ALTER TABLE  `" . $table_name . "` ADD  `" . $blog_id . "` BIGINT( 20 ) NULL ,
			ADD UNIQUE `blog_u_" . $blog_id . "` (`" . $blog_id . "`);";

			$wpdb->query($sql);
	   	}

	   	delete_site_transient('wordpress_multilingual_multisite_blogs_data');
	}

	/**
	 * [remove_blog_from_translation_table description]
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $blog_id [description]
	 * @param   [type]                       $drop    [description]
	 * @return  [type]                                [description]
	 */
	public function remove_blog_from_translation_table($blog_id, $drop)
	{
	   	global $wpdb, $wordpress_multilingual_multisite_delete_running;

	   	$wordpress_multilingual_multisite_delete_running = true;

	   	$tables = array('posts_translations', 'terms_translations');
	   	foreach ($tables as $table) {
		   	$table_name = $wpdb->base_prefix . $table;

			$sql = "ALTER TABLE  `" . $table_name . "` DROP  `" . $blog_id . "`;";
			$wpdb->query($sql);

			$sql = "ALTER TABLE  `" . $table_name . "` DROP  INDEX `blog_u_" . $blog_id . "`;";
			$wpdb->query($sql);
	   	}

	   	delete_site_transient('wordpress_multilingual_multisite_blogs_data');
	}

	/**
	 * [remove_blog_from_translation_table description]
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $blog_id [description]
	 * @param   [type]                       $drop    [description]
	 * @return  [type]                                [description]
	 */
	public function get_all_items($args = array())
	{
	   	global $wpdb;

		$sql = "SELECT * FROM  `" . $this->table . "`;";

		$items = $wpdb->query($sql);

		return $items;
	}

	/**
	 * [remove_blog_from_translation_table description]
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $blog_id [description]
	 * @param   [type]                       $drop    [description]
	 * @return  [type]                                [description]
	 */
	public function get_translated_items_by_item_id($blog_id, $item_id, $args = array())
	{
	   	global $wpdb, $blogs_data;

	   	$results = array();

	   	if(get_post_type($item_id) == "product_variation") {

	   		$sku = get_post_meta($item_id, '_sku', true);
			foreach ($blogs_data as $blog_id => $blog_data) {

				switch_to_blog($blog_id);

		        $productVariationId = wc_get_product_id_by_sku($sku);
		        if($productVariationId) {
		        	$results[$blog_id] = $productVariationId;
		        }

		        restore_current_blog();

	        }

	        if(!empty($results)) {
	        	return $results;
	        }

	   	} else {
	   		$results = $wpdb->get_results("SELECT * FROM  `" . $this->table . "` WHERE `" . $blog_id . "` = '" . $item_id . "';", ARRAY_A);	
	   	}		

		if(isset($results[0]) && !empty($results[0])) {
			return $results[0];
		}

		return false;
	}

	/**
	 * [remove_blog_from_translation_table description]
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $blog_id [description]
	 * @param   [type]                       $drop    [description]
	 * @return  [type]                                [description]
	 */
	public function get_translated_items_by_translated_items($translated_items)
	{
	   	global $wpdb;

	   	foreach ($translated_items as $blog_id => $item_id) {

	   		if($blog_id == "type") {
	   			continue;
	   		}

	   		$results = $wpdb->get_results("SELECT * FROM  `" . $this->table . "` WHERE `" . $blog_id . "` = '" . $item_id . "';", ARRAY_A);
			if(isset($results[0]) && !empty($results[0])) {
				return $results[0];
			}
	   	}

		return false;
	}

	/**
	 * [remove_blog_from_translation_table description]
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $blog_id [description]
	 * @param   [type]                       $drop    [description]
	 * @return  [type]                                [description]
	 */
	public function update_translated_items($id, $blog_id, $item_id, $translated_items)
	{
	   	global $wpdb;

	   	foreach ($translated_items as $blog_id_ => $item_id_) {

	   		if($blog_id_ == "type") {
	   			continue;
	   		}

	   		if(empty($item_id_)) {
	   			$translated_items[$blog_id_] = NULL;
	   		}

	   		$wpdb->update($this->table, array($blog_id_ => null), array($blog_id_ => $item_id_));
	   	}
	   	
	   	$wpdb->update($this->table, $translated_items, array('id' => $id), array( '%s', '%s'));
	}

	/**
	 * [remove_blog_from_translation_table description]
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $blog_id [description]
	 * @param   [type]                       $drop    [description]
	 * @return  [type]                                [description]
	 */
	public function insert_translated_items($blog_id, $item_id, $translated_items)
	{
	   	global $wpdb;

	   	foreach ($translated_items as $blog_id_ => $item_id_) {

	   		if($blog_id_ == "type") {
	   			continue;
	   		}

	   		if($blog_id_ == $this->current_blog_id) {
	   			continue;
	   		}

	   		if(empty($item_id_)) {
	   			$translated_items[$blog_id_] = NULL;
	   		}
	   		
	   		$wpdb->update($this->table, array($blog_id_ => null), array($blog_id_ => $item_id_));
	   	}

	   	$wpdb->insert($this->table, $translated_items, array( '%s', '%s'));

		$my_id = $wpdb->insert_id;
	}

	/**
	 * [remove_blog_from_translation_table description]
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $blog_id [description]
	 * @param   [type]                       $drop    [description]
	 * @return  [type]                                [description]
	 */
	public function get_translated_items_by_type($type, $blog_ids = array())
	{
	   	global $wpdb;

	   	if(empty($type)) {
	   		return false;
	   	}

	   	$select = "";
	   	if(!empty($blog_ids)) {

	   		foreach ($blog_ids as $blog_id) {
	   			$select .= '`' . $blog_id . '`,';
	   		}
	   		$select = rtrim( $select, ',');
	   	} else {
	   		$select = '*';
	   	}

   		$results = $wpdb->get_results("SELECT " . $select . " FROM  `" . $this->table . "` WHERE `type` = '" . $type . "';", ARRAY_A);
		if(isset($results) && !empty($results)) {
			return $results;
		}

		return false;
	}

	/**
	 * [remove_blog_from_translation_table description]
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $blog_id [description]
	 * @param   [type]                       $drop    [description]
	 * @return  [type]                                [description]
	 */
	public function get_post_types()
	{
	   	global $wpdb;

   		$results = $wpdb->get_results("SELECT DISTINCT(type) FROM  `" . $this->table . "`", ARRAY_A);
		if(isset($results) && !empty($results)) {
			return $results;
		}

		return false;
	}
	
}